"""
Storage Handling for spelling
"""

import json

import dataset


class StorageHandler(object):
    """
    Simple storage implementation.
    """

    def __init__(self, storage_path):
        self.con = self.get_db(storage_path)
        self.table = self.con["storage"]

    @staticmethod
    def get_db(storage_path):
        """
        Open storage connection.
        """
        if storage_path is None:
            return dataset.connect("sqlite:///:memory:")
        return dataset.connect("sqlite:///%s" % (storage_path,))

    def load_word_count(self):
        """
        Retrieve stored word count.
        """
        table = self.table
        row = table.find_one(name="word_count")
        if not row:
            return {}
        return json.loads(row["value"])

    def save_word_count(self, value):
        """
        Save word count to storage.
        """
        table = self.table
        table.upsert({"name": "word_count", "value": json.dumps(value)}, keys=["name"])


def get_store(storage_path):
    """
    Return the handler for storing details.
    """
    return StorageHandler(storage_path)



"""
Main invocation for spelling check
"""
from __future__ import absolute_import, division, print_function

import pathlib

import pyspelling

from spelling.config import get_config_context_manager
from spelling.store import get_store


def check(display_context, display_summary, config, storage_path, fobj):
    """
    Execute the invocation
    """
    workingpath = pathlib.Path(".").resolve()
    success = True
    check_iter_output = check_iter(
        display_context, display_summary, config, storage_path, workingpath
    )
    for output in check_iter_output:
        print(output, file=fobj)
        success = False
    if success:
        print("Spelling check passed :)", file=fobj)
    return success


def check_iter(display_context, display_summary, config, storage_path, workingpath):
    """
    Execute the invocation
    """
    all_results = run_spell_check(config, storage_path, workingpath)
    fail = False
    misspelt = set()
    for results in all_results:
        if results.error:
            fail = True
            yield "ERROR: %s -- %s" % (results.context, results.error)
        elif results.words:
            fail = True
            misspelt.update(results.words)
            if display_context:
                yield "Misspelled words:\n<%s> %s" % (results.category, results.context)
                yield "-" * 80
                for word in results.words:
                    yield word
                yield "-" * 80
                yield ""

    if fail:
        yield "!!!Spelling check failed!!!"
        if display_summary:
            yield "\n".join(sorted(misspelt))


def run_spell_check(config, storage_path, workingpath):
    """
    Perform the spell check and keep a record of spelling mistakes.
    """
    with get_config_context_manager(workingpath, config) as ctxt:
        all_results = list(
            pyspelling.spellcheck(
                ctxt.config,
                names=[],
                groups=[],
                binary="",
                sources=[],
                verbose=0,
                debug=False,
            )
        )
    storage = get_store(storage_path)
    wordcount = storage.load_word_count()
    for results in all_results:
        for word in results.words:
            wordcount[word] = wordcount.get(word, 0) + 1
    storage.save_word_count(wordcount)
    return all_results


# vim: set ft=python:



#!/usr/bin/env python
"""
Module load handler for execution via:

python -m spelling
"""
from __future__ import absolute_import, division, print_function

import logging
import os
import pathlib
import sys

import click

from spelling.check import check_iter
from spelling.version import __version__

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS, invoke_without_command=True)
@click.version_option(version=__version__)
@click.pass_context
@click.option("--display-context/--no-display-context", default=True)
@click.option("--display-summary/--no-display-summary", default=True)
@click.option("--config", default=None)
@click.option("--storage-path", default=None)
@click.option("--working-path", default=None)
def main(  # pylint: disable=too-many-arguments,bad-continuation
    ctxt, display_context, display_summary, config, storage_path, working_path
):
    """
    Used to conveniently invoke spell checking with sensible defaults and
    command line arguments to change the behavior.
    """
    if ctxt.invoked_subcommand is None:
        run_invocation(
            display_context, display_summary, config, storage_path, working_path
        )


@main.command()
@click.option("--display-context/--no-display-context", default=True)
@click.option("--display-summary/--no-display-summary", default=True)
@click.option("--config", default=None)
@click.option("--storage-path", default=None)
@click.option("--working-path", default=None)
def invoke(display_context, display_summary, config, storage_path, working_path):
    """
    Invoke the spell checker
    """
    run_invocation(display_context, display_summary, config, storage_path, working_path)


def run_invocation(  # pylint: disable=bad-continuation
    display_context, display_summary, config, storage_path, working_path
):
    """
    Call spell checker
    """
    logging.basicConfig()
    try:
        success = True
        if working_path is None:
            working_path = pathlib.Path(os.getcwd()).resolve()
        msg_iter = check_iter(
            display_context=display_context,
            display_summary=display_summary,
            config=config,
            storage_path=storage_path,
            workingpath=working_path,
        )
        for msg in msg_iter:
            print(msg)
            success = False
        if not success:
            sys.exit(1)
        print("Spelling check passed :)")
    except Exception:
        logging.exception("Error during processing.")
        raise


if __name__ == "__main__":
    main()  # pylint: disable=no-value-for-parameter
# vim: set ft=python:
